###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.2 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
from DateTime import DateTime
import Acquisition
import AccessControl.Role
import OFS.Folder
import base_item
import utils
import DateTime

calendar_event_add_redirect = HTMLFile('calendar_event_add_redirect',
									   globals())
manage_add_calendar_event_form = HTMLFile('calendar_event_add', globals())

def manage_add_calendar_event(self, title='', note='', alarm=0, start=0, end=0,
							  due=0, priority=0, status=0,
							  REQUEST=None, RESPONSE=None):
	"Add an instance of the calendar_event class."

	new_id = str(self.get_unique_id())

	if REQUEST:

		expand_hour = lambda x: x * 60 * 60
		expand_minute = lambda x: x * 60

		alarm = expand_hour(REQUEST['alarm_hour']) + \
				expand_minute(REQUEST['alarm_minute'])
		start = expand_hour(REQUEST['start_hour']) + \
				expand_minute(REQUEST['start_minute'])
		end = expand_hour(REQUEST['end_hour']) + \
			  expand_minute(REQUEST['end_minute'])
		due = expand_hour(REQUEST['due_hour']) + \
			  expand_minute(REQUEST['due_minute'])

		title = REQUEST['title']
		note = REQUEST['note']
		
		
	conflicting_events = self.check_timespan(
		self.getParentNode().getParentNode().id,
		self.getParentNode().id, self.id,
		start, end)

	if conflicting_events:
		
		raise 'ConflictingEvents', str(map(lambda x: (x.start, x.end, x.id),
										   conflicting_events)) + \
										   str(start) + '::' + str(end)
	
	self._setObject(new_id, calendar_event(
		new_id, title, note, alarm, start, end, due, priority, status))

	if REQUEST is not None and RESPONSE is not None:

		RESPONSE.redirect(REQUEST['URL1'])


class calendar_event(
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'CalendarEvent'
	icon = 'misc_/Calendar/Folder_icon.gif'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'note', 'type':'text'},
		{'id':'alarm', 'type':'int'},
		{'id':'start', 'type':'int'},
		{'id':'end', 'type':'int'},
		{'id':'due', 'type':'int'},
		{'id':'priority', 'type':'int'},
		{'id':'status', 'type':'int'},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View CalendarEvents',
		 ('',)),
		)

	index_html = HTMLFile('calendar_event_index', globals())
	display = HTMLFile('calendar_event_display', globals())

	base_edit = HTMLFile('calendar_event_edit', globals())

	calendar_event_edit_redirect = HTMLFile('calendar_event_edit_redirect',
											globals())

	priorities = {
		0:'<span style="color: green;">low</span>',
		1:'<span style="color: yellow;">medium</span>',
		2:'<span style="color: red;">high</span>'}

	statuses = {
		0:'completed',
		1:'pending',
		2:'cancelled'
		}

	make_hour_minutes_select = utils.make_hour_minutes_select


	def __init__(self, id, title='', note='', alarm=0,
				 start=0, end=0, due=0,
				 priority=0, status='0'):
		"Initialize the class instance"
		
		self.id = id
		self.title = title
		self.note = note
		self.alarm = alarm
		self.start = start
		self.end = end
		self.due = due
		self.priority = priority
		self.status = status


	def render_priority(self):
		"""Renders given priority."""

		return self.priorities[self.priority]


	def render_start(self):
		"""Renders the start time."""

		return utils.format_time(self.start)


	def render_end(self):
		"""Renders the end time."""

		return utils.format_time(self.end)


	def render_due(self):
		"""Renders the time the event is due."""

		return utils.format_time(self.due)


	def render_status(self):
		"""Renders the status of the event."""

		return self.statuses[self.status]
	

	def manage_reschedule(self, REQUEST=None, RESPONSE=None):
		"""Moves an event between days."""

		year = utils.pad_integer(REQUEST['year'])
		month = utils.pad_integer(REQUEST['month'])
		day = utils.pad_integer(REQUEST['day'])

		try:

			DateTime.DateTime('%s-%s-%s' % (year, month, day))

		except DateTime.SyntaxError:

			raise 'InvalidDate', 'Sorry, the date you specified is invalid'

		calendar = self.get_calendar()

		try:

			calendar[year]

		except:

			calendar.manage_add_calendar_year(year)


		conflicting_events = self.check_timespan(year, month, day,
												 self.start, self.end)

		if conflicting_events:

			raise 'ConflictingEvents', 'One or more events on the requested day conflict with this event'

		event = self.getParentNode().manage_cutObjects(self.id)
		calendar[year][month][day].manage_pasteObjects(cb_copy_data=event)

		if RESPONSE:

			RESPONSE.redirect(calendar[year][month][day].absolute_url())


default__class_init__(calendar_event)
