###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.3 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import news_group_storage
import base_item
import nntplib
import string


news_host_add_redirect = HTMLFile('news_host_add_redirect', globals())
manage_add_news_host_form = HTMLFile('news_host_add', globals())


def manage_add_news_host(self, id='', server='', port='',
						 REQUEST=None, RESPONSE=None):
    "Add an instance of the news_host class"

    if not id:

		id = str(self.get_unique_id())

    self._setObject(id, news_host(id, server, port))

    if REQUEST is not None and RESPONSE is not None:

        RESPONSE.redirect(self.absolute_url())


class news_host(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	product_name = 'NewsHost'

	meta_type = 'NewsHost'
	icon = 'misc_/Transform/Folder_icon.gif'

	_properties = (
		{'id':'server', 'type':'string'},
		{'id':'port', 'type':'int'},
		{'id':'active_list', 'type':'lines'},
		)

	meta_types = (
		{'name':'NewsGroupStorage',
		 'action':'manage_add_news_group_storage_form'},
		)

	manage_add_news_group_storage_form = news_group_storage.manage_add_news_group_storage_form
	manage_add_news_group_storage = news_group_storage.manage_add_news_group_storage
	news_group_storage_add_redirect = news_group_storage.news_group_storage_add_redirect

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add NewsGroupStorage',
		 ('manage_add_news_group_storage',
		  'manage_add_news_group_storage_form')),
		('View NewsHosts',
		 ('',)),
		)

	index_html = HTMLFile('news_host_index', globals())


	def __init__(self, id, server='', port=None, active_list=''):
		"Initialize the class instance"

		self.id = id
		self.server = server
		self.port = port
		self.active_list = active_list

		self.counter = 0

		try:

			self.index_object()

		except AttributeError:

			# No ZCatalog in acquisition path..

			pass

		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_header',
									 'standard_html_header',
									 open('%s/Products/%s/standard_html_header.dtml' % \
										  (SOFTWARE_HOME, self.product_name)).read())
										  
		OFS.DTMLMethod.addDTMLMethod(self, 'standard_html_footer',
									 'standard_html_footer',
									 open('%s/Products/%s/standard_html_footer.dtml' % \
										  (SOFTWARE_HOME, self.product_name)).read())

		OFS.DTMLMethod.addDTMLMethod(self, 'standard_css', 'standard_css',
									 open('%s/Products/%s/standard_css.dtml' % \
										  (SOFTWARE_HOME, self.product_name)).read())		

		connection = self.refresh()


	def get_unique_id(self):
		"""Returns a unique identifier.

		The identifier may or may not be globally unique."""

		try:

			return self.getParentNode().get_unique_id()

		except AttributeError:

			self.counter = self.counter + 1

			return self.counter


	def refresh(self):
		"""Opens a connection."""

		if self.port:

			return nntplib.NNTP(self.server, self.port)

		else:

			return nntplib.NNTP(self.server)


	def get_subscribed_newsgroups(self):
		"""Returns a list of subscribed newsgroups."""

		newsgroups = []

		map(newsgroups.append, self.objectValues('NewsGroupStorage'))

		return newsgroups


	def retrieve_group(self, newsgroup, run_as_thread=1):
		"""Retrieves messages for a given group."""

		self._retrieve_group(newsgroup)
		return

		# FIXME

		if run_as_thread:

			threading._start_new_thread(self._retrieve_group, (newsgroup,))

		else:

			self._retrieve_group(newsgroup)


	def _retrieve_group(self, newsgroup):
		"""Retrieves messages for a given group."""

		#raise 'RetrievingGroup', newsgroup

		connection = self.refresh()

		news_group_storage = self[newsgroup]

		dummy, dummy, first, last, dummy = connection.group(newsgroup)

		first = int(first)
		last = int(last)


		# If there are new messages.

		if last > news_group_storage.last_retrieved_message:

			# If the first (available) message on the newsserver
			# is more recent than then last message we retrieved
			# we can safely skip our own counter and start retrieving
			# based upon the message number supplied by the server.

			if first > news_group_storage.last_retrieved_message:

				message_retrieval_start = first

			else:

				message_retrieval_start = news_group_storage.last_retrieved_message

			for message_number in range(message_retrieval_start,
										last + 1):

				try:
					
					# Should we add more program logic here, WRT to
					# status codes, etc?

					article = string.join(
						connection.article(str(message_number))[-1],'\n')

					Products.RFC822Message.util.store_unparsed_message(
						news_group_storage, article, readonly=1)

					news_group_storage.last_retrieved_message = message_number

				except nntplib.error_temp, value:

					# Add program logic (handling of status, etc)
					# FIXME

					pass

		else:

			# No new messages for given group.

			raise 'Yay', 'No new messages for %s' % newsgroup

			pass
				

	def retrieve(self, RESPONSE=None):
		"""Retrieves new messages."""

		for newsgroup in self.get_subscribed_newsgroups():

			self.retrieve_group(newsgroup.id)

		if RESPONSE is not None:

			RESPONSE.redirect(self.absolute_url())


	def manage_delete(self, ids=[], RESPONSE=None):
		"""Deletes the messages specified."""

		self.manage_delObjects(ids=ids)

		if RESPONSE:

			RESPONSE.redirect(self.absolute_url())


	def post(self, message):
		"""Posts a message."""

		connection = self.refresh()

		return connection.post(
			Products.RFC822Message.util.my_string_io(message))


default__class_init__(news_host)

