###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.2 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder

todoAdd2 = HTMLFile('todoAdd2', globals())
manage_addTodoForm = HTMLFile('todoAdd', globals())


def manage_addTodo(self, id='', title='', short_body='', body='', priority='',
				   REQUEST=None, RESPONSE=None):
    "Add an instance of the Todo class"

    # FIXME -- we don't want this mess here..x

    if type(id) is type(1):

		id = str(id)

    else:

		if not id:

			id = str(self.get_unique_id())
			
    self._setObject(id, Todo(id, title, short_body, body, priority))

    if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url())


class Todo(
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager
	):
	"Documentation string"

	meta_type = 'Todo'
	icon = 'misc_/TodoFolder/Folder_icon.gif'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'short_body', 'type':'string'},
		{'id':'body', 'type':'text'},
		{'id':'priority', 'type':'int'},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View Todos',
		 ('',)),
		)

	meta_types = (
		{'name':'Todo', 'action':'manage_addTodoForm'},
		)

	manage_addTodoForm = manage_addTodoForm
	manage_addTodo = manage_addTodo
	todoAdd2 = todoAdd2	

	index_html = HTMLFile('todoIndex', globals())

	todo_edit = HTMLFile('todo_edit', globals())

	split = HTMLFile('todo_split', globals())

	priorities = {
		0: 'Low',
		1: 'Medium',
		2: 'High'
		}


	def __init__(self, id, title='', short_body='', body='', priority=''):
		"Initialize the class instance"

		self.id = id
		self.title = title
		self.short_body = short_body
		self.body = body
		self.priority = priority

		# We use this to give the todos unique titles.

		#self.todo_level = 0


	#def get_level(self):
	#	"""Renders the current level and increments the counter."""

	#	self.todo_level = self.todo_level + 1

	#	return str(self.todo_level)
		

	def split_post(self, REQUEST=None, RESPONSE=None):
		"""Splits up a todo into to separate todos."""

		split_type = REQUEST['split_type']

		if split_type == 'sub-todo':

			container = self

		elif split_type == 'new-todo':

			container = self.getParentNode()

		else:

			raise 'UnknownSplitType', split_type

		for title in REQUEST['new_title']:

			manage_addTodo(container, self.get_unique_id(),
						   title, self.short_body, self.body,
						   self.priority)


		if split_type == 'sub-todo':

			RESPONSE.redirect(self.absolute_url())

		elif split_type == 'new-todo':

			RESPONSE.redirect(self.getParentNode().absolute_url())


	def render_priority(self):
		"""Returns a string indicating the priority."""

		return self.priorities[self.priority]
	

	def display(self):
		"""Returns a view suitable in a larger picture."""

		return """<table border="0">
		<tr><th>(Priority %s)</th><td>%s</td></tr>
		</table>
		""" % (self.render_priority(), self.short_body)


	def delete(self, REQUEST=None, RESPONSE=None):
		"""Deletes self; and redirects to containing object."""

		del self

		if RESPONSE:

			RESPONSE.redirect(REQUEST['HTTP_REFERRER'])


default__class_init__(Todo)

